@@ -184,6 +184,7 @@ func (m *mockGitClient) StashList() (string, error) { return "", nil }
 func (m *mockGitClient) StashShow(_ string) error   { return nil }
 func (m *mockGitClient) StashApply(_ string) error  { return nil }
 func (m *mockGitClient) StashPop(_ string) error    { return nil }
+func (m *mockGitClient) StashPush(_ string) error   { return nil }
 func (m *mockGitClient) StashDrop(_ string) error   { return nil }
 func (m *mockGitClient) StashClear() error          { return nil }
 
@@ -456,12 +457,21 @@ func TestCmd_Help(t *testing.T) {
 		}
 	}()
 
-	cmd.Help()
+	cmd.Help(nil)
 
 	// Verify that help was actually called (buffer should have content)
 	if buf.Len() == 0 {
 		t.Error("Help() should produce output")
 	}
+
+	buf.Reset()
+	cmd.Help([]string{"branch"})
+	if buf.Len() == 0 {
+		t.Error("Help(command) should produce output")
+	}
+	if !strings.Contains(buf.String(), "ggc branch") {
+		t.Error("Help(command) should render branch usage")
+	}
 }
 
 func TestCmd_Branch(t *testing.T) {
