@@ -3,10 +3,14 @@ package templates
 
 import (
 	"bytes"
+	"fmt"
 	"os"
+	"strings"
 	"text/template"
 
 	"golang.org/x/term"
+
+	commandregistry "github.com/bmf-san/ggc/v7/cmd/command"
 )
 
 // HelpData contains data for help message templates.
@@ -25,6 +29,12 @@ Usage:
   ggc <command> [subcommand] [options]
 
 Main Commands:
+{{if .Categories}}
+  {{range .Categories}}{{.Name}}:
+  {{range .Commands}}  {{.Display}}
+  {{end}}
+  {{end}}
+  {{else}}
   ggc help                    Show help message
   ggc add <file>              Stage file(s)
   ggc add .                   Stage all changes
@@ -76,16 +86,15 @@ Main Commands:
   ggc restore <commit> <file> Restore file from specific commit
   ggc version                 Show current ggc version
   ggc config                  Manage ggc configuration
+  ggc debug-keys              Debug keybinding issues and capture key sequences
   ggc hook                    Manage Git hooks
   ggc reset                   Reset and clean
   ggc stash                   Stash changes
   ggc status                  Show the working tree status
 
-Notes:
-  - Unified syntax: no option flags (-/--) — use subcommands and words.
-  - To pass a literal that starts with '-', use the '--' separator:
-      ggc commit -- - fix leading dash
-`
+  {{end}}Notes:
+  {{range .Notes}}  - {{.}}
+  {{end}}`
 
 	commandHelpTemplate = `{{.Logo}}
 Usage: {{.Usage}}
@@ -101,7 +110,20 @@ Examples:
 
 // MainHelpData contains data for main help message.
 type MainHelpData struct {
-	Logo string
+	Logo       string
+	Categories []helpCategory
+	Notes      []string
+}
+
+type helpCategory struct {
+	Name     string
+	Commands []helpCommand
+}
+
+type helpCommand struct {
+	Usage   string
+	Summary string
+	Display string
 }
 
 func selectLogo() string {
@@ -122,7 +144,12 @@ func RenderMainHelp() (string, error) {
 
 	var buf bytes.Buffer
 	data := MainHelpData{
-		Logo: selectLogo(),
+		Logo:       selectLogo(),
+		Categories: buildMainHelpCategories(),
+		Notes: []string{
+			"Unified syntax: no option flags (-/--) — use subcommands and words.",
+			"To pass a literal that starts with '-', use the '--' separator: ggc commit -- - fix leading dash",
+		},
 	}
 
 	if err := tmpl.Execute(&buf, data); err != nil {
@@ -132,6 +159,105 @@ func RenderMainHelp() (string, error) {
 	return buf.String(), nil
 }
 
+func buildMainHelpCategories() []helpCategory {
+	orderedCats := commandregistry.OrderedCategories()
+
+	categoryCommands := make(map[commandregistry.Category][]helpCommand)
+
+	visibleCommands := commandregistry.VisibleCommands()
+	for i := range visibleCommands {
+		cmd := &visibleCommands[i]
+		categoryCommands[cmd.Category] = append(categoryCommands[cmd.Category], helpCommandsFor(cmd)...)
+	}
+
+	var categories []helpCategory
+	for _, cat := range orderedCats {
+		commands := categoryCommands[cat]
+		if len(commands) == 0 {
+			continue
+		}
+
+		maxUsage := 0
+		for _, cmd := range commands {
+			if len(cmd.Usage) > maxUsage {
+				maxUsage = len(cmd.Usage)
+			}
+		}
+
+		for i := range commands {
+			usage := commands[i].Usage
+			summary := commands[i].Summary
+			if summary != "" {
+				commands[i].Display = fmt.Sprintf("    %-*s %s", maxUsage, usage, summary)
+			} else {
+				commands[i].Display = fmt.Sprintf("    %s", usage)
+			}
+		}
+
+		categories = append(categories, helpCategory{
+			Name:     string(cat),
+			Commands: commands,
+		})
+	}
+
+	return categories
+}
+
+func helpCommandsFor(info *commandregistry.Info) []helpCommand {
+	var entries []helpCommand
+	if len(info.Subcommands) == 0 {
+		usage := firstUsage(info.Usage, "ggc "+info.Name)
+		if shouldIncludeUsage(usage) {
+			entries = append(entries, helpCommand{Usage: usage, Summary: info.Summary})
+		}
+		return entries
+	}
+
+	for _, sub := range info.Subcommands {
+		if sub.Hidden {
+			continue
+		}
+		usage := firstUsage(sub.Usage, "ggc "+sub.Name)
+		if !shouldIncludeUsage(usage) {
+			continue
+		}
+		entries = append(entries, helpCommand{Usage: usage, Summary: sub.Summary})
+	}
+
+	return dedupeHelpCommands(entries)
+}
+
+func firstUsage(usages []string, fallback string) string {
+	for _, usage := range usages {
+		trimmed := strings.TrimSpace(usage)
+		if trimmed != "" {
+			return trimmed
+		}
+	}
+	return fallback
+}
+
+func shouldIncludeUsage(usage string) bool {
+	return strings.HasPrefix(usage, "ggc ")
+}
+
+func dedupeHelpCommands(commands []helpCommand) []helpCommand {
+	if len(commands) <= 1 {
+		return commands
+	}
+	seen := make(map[string]struct{}, len(commands))
+	var result []helpCommand
+	for _, cmd := range commands {
+		key := cmd.Usage + "\n" + cmd.Summary
+		if _, ok := seen[key]; ok {
+			continue
+		}
+		seen[key] = struct{}{}
+		result = append(result, cmd)
+	}
+	return result
+}
+
 // RenderCommandHelp renders help message for a specific command.
 func RenderCommandHelp(data HelpData) (string, error) {
 	tmpl, err := template.New("commandHelp").Parse(commandHelpTemplate)
