@@ -6,6 +6,7 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"sort"
 	"strconv"
 	"strings"
 	"sync/atomic"
@@ -183,12 +184,28 @@ type UIState struct {
 
 // UpdateFiltered updates the filtered commands based on current input using fuzzy matching
 func (s *UIState) UpdateFiltered() {
-	s.filtered = []CommandInfo{}
 	input := strings.ToLower(s.input)
-	for _, cmd := range commands {
-		cmdLower := strings.ToLower(cmd.Command)
-		if fuzzyMatch(cmdLower, input) {
-			s.filtered = append(s.filtered, cmd)
+	if input == "" {
+		s.filtered = make([]CommandInfo, len(commands))
+		copy(s.filtered, commands)
+	} else {
+		type match struct {
+			info  CommandInfo
+			score matchScore
+		}
+		matches := make([]match, 0, len(commands))
+		for _, cmd := range commands {
+			cmdLower := strings.ToLower(cmd.Command)
+			if ok, score := fuzzyMatchScore(cmdLower, input); ok {
+				matches = append(matches, match{info: cmd, score: score})
+			}
+		}
+		sort.SliceStable(matches, func(i, j int) bool {
+			return matches[i].score.less(matches[j].score)
+		})
+		s.filtered = make([]CommandInfo, len(matches))
+		for i, match := range matches {
+			s.filtered[i] = match.info
 		}
 	}
 	// Reset selection if out of bounds
@@ -267,24 +284,118 @@ func (s *UIState) IsInSearchMode() bool {
 // fuzzyMatch performs fuzzy matching between text and pattern
 // Returns true if all characters in pattern appear in text in order (but not necessarily consecutive)
 func fuzzyMatch(text, pattern string) bool {
+	matched, _ := fuzzyMatchScore(text, pattern)
+	return matched
+}
+
+// fuzzyMatchScore returns whether the pattern matches the text and a relevance score for sorting results.
+// Lower scores indicate a tighter, earlier match.
+func fuzzyMatchScore(text, pattern string) (bool, matchScore) {
 	if pattern == "" {
-		return true
+		return true, matchScore{length: len([]rune(text))}
 	}
 
 	textRunes := []rune(text)
 	patternRunes := []rune(pattern)
 
+	matched, meta := matchPattern(textRunes, patternRunes)
+	if !matched {
+		return false, matchScore{}
+	}
+
+	trailing := len(textRunes) - meta.lastIndex - 1
+	continuation := continuationPenalty(textRunes, meta.lastIndex)
+	score := matchScore{
+		first:        meta.firstIndex,
+		gap:          meta.gapScore,
+		trailing:     trailing,
+		continuation: continuation,
+		length:       len(textRunes),
+	}
+
+	return true, score
+}
+
+type matchMetadata struct {
+	firstIndex int
+	lastIndex  int
+	gapScore   int
+}
+
+func matchPattern(textRunes, patternRunes []rune) (bool, matchMetadata) {
+	meta := matchMetadata{
+		firstIndex: -1,
+		lastIndex:  -1,
+	}
+
 	textIdx := 0
 	patternIdx := 0
 
 	for textIdx < len(textRunes) && patternIdx < len(patternRunes) {
 		if textRunes[textIdx] == patternRunes[patternIdx] {
+			if meta.firstIndex == -1 {
+				meta.firstIndex = textIdx
+			}
+			if meta.lastIndex != -1 {
+				meta.gapScore += textIdx - meta.lastIndex - 1
+			}
+			meta.lastIndex = textIdx
 			patternIdx++
 		}
 		textIdx++
 	}
 
-	return patternIdx == len(patternRunes)
+	if patternIdx != len(patternRunes) {
+		return false, meta
+	}
+
+	return true, meta
+}
+
+func continuationPenalty(textRunes []rune, lastMatchIdx int) int {
+	if lastMatchIdx < 0 || lastMatchIdx+1 >= len(textRunes) {
+		return 0
+	}
+
+	nextIdx := lastMatchIdx + 1
+	spaceSkipped := false
+	for nextIdx < len(textRunes) && textRunes[nextIdx] == ' ' {
+		spaceSkipped = true
+		nextIdx++
+	}
+
+	if spaceSkipped && nextIdx < len(textRunes) && (unicode.IsLetter(textRunes[nextIdx]) || unicode.IsDigit(textRunes[nextIdx])) {
+		return 1
+	}
+
+	return 0
+}
+
+type matchScore struct {
+	first        int
+	gap          int
+	trailing     int
+	continuation int
+	length       int
+}
+
+func (m matchScore) less(other matchScore) bool {
+	if m.first != other.first {
+		return m.first < other.first
+	}
+	if m.gap != other.gap {
+		return m.gap < other.gap
+	}
+	if m.continuation != other.continuation {
+		return m.continuation < other.continuation
+	}
+	if m.trailing != other.trailing {
+		return m.trailing < other.trailing
+	}
+	if m.length != other.length {
+		return m.length < other.length
+	}
+	return false
 }
 
 // MoveUp moves selection up
